require 'rails_helper'

RSpec.describe BenchmarkConfiguration do
  context "repository" do
    context "for_benchmark_infos" do
      subject do
        -> (infos) { BenchmarkConfiguration.for_benchmark_infos(infos) }
      end

      it "returns all configurations having batch with results for infos" do
        linux = OperatingSystem.create!(name: "Linux 3.x (i386)")
        win = OperatingSystem.create!(name: "Windows XP (i386)")
        smalltalk = Language.create!(name: "Smalltalk")
        stx = Runtime.create!(name: "Smalltalk/X", language: smalltalk)
        pharo = Runtime.create!(name: "Pharo", language: smalltalk)
        stx_on_linux = BenchmarkConfiguration.create!(language: smalltalk,
                                                      runtime: stx,
                                                      operating_system: linux)
        pharo_on_win = BenchmarkConfiguration.create!(language: smalltalk,
                                                      runtime: pharo,
                                                      operating_system: win)
        pharo_on_linux = BenchmarkConfiguration.create!(language: smalltalk,
                                                        runtime: pharo,
                                                        operating_system: linux)
        deltablue = BenchmarkInfo.create!(name: "DeltaBlue",
                                          benchmark_class: "DeltaBlue",
                                          benchmark_selector: "run")
        ackermann = BenchmarkInfo.create!(name: "Ackermann",
                                          benchmark_class: "BenchmarkMicro",
                                          benchmark_selector: "ackermann")
        stx_today = BenchmarkBatch.create!(performed_at: Date.today,
                                           benchmark_configuration: stx_on_linux)
        pharo_today = BenchmarkBatch.create!(performed_at: Date.today,
                                             benchmark_configuration: pharo_on_win)
        stx_yesterday = BenchmarkBatch.create!(performed_at: Date.yesterday,
                                               benchmark_configuration: stx_on_linux)
        pharo_yesterday = BenchmarkBatch.create!(performed_at: Date.yesterday,
                                                 benchmark_configuration: pharo_on_win)
        res1 = BenchmarkResult.create!(benchmark_info: deltablue,
                                       benchmark_batch: stx_today)
        res2 = BenchmarkResult.create!(benchmark_info: ackermann,
                                       benchmark_batch: stx_today)
        res3 = BenchmarkResult.create!(benchmark_info: deltablue,
                                       benchmark_batch: stx_yesterday)
        res4 = BenchmarkResult.create!(benchmark_info: ackermann,
                                       benchmark_batch: stx_yesterday)
        res5 = BenchmarkResult.create!(benchmark_info: deltablue,
                                       benchmark_batch: pharo_today)
        res6 = BenchmarkResult.create!(benchmark_info: ackermann,
                                       benchmark_batch: pharo_today)
        res7 = BenchmarkResult.create!(benchmark_info: deltablue,
                                       benchmark_batch: pharo_yesterday)
        res8 = BenchmarkResult.create!(benchmark_info: ackermann,
                                       benchmark_batch: pharo_yesterday)

        expect(subject[[deltablue.id, ackermann.id]]).
          to match_array([stx_on_linux, pharo_on_win])
      end
    end
  end
end
